'******************************************************
'Hidenori Shoji (TOKAI OPTICAL CO., LTD)
'Ver.08:2006/5/2
'
'Function:
'Read of UDS file
'******************************************************
Option Explicit
DefInt i-n
DefSng A-H, O-Z

'----- Global variables -----
Const FNAME_INI As String = "\T_Ini_Read_UDS.fbi" 	'Initialization file name
Const FNAME_SPEC As String  = "\T_Read_UDS.csv" 		'CSV file name for "Spectra"

'----- Structure -----
'Measurement data structure
Public Type SpFileData
    File_Path As String
    Title As String
    Comment As String
    Num As Integer   	' data number
    W_begin As Double 	' nm
    W_end As Double   	' nm
    W_step As Double  	' nm
    Data() As Double 	' Spectral data %
    DataType As Integer	' data mode@0:trans  4:ref
End Type
Dim Sp As SpFileData

'Target data structure
Type TargetData
	AOI As Single 			'AOI[deg]
	Polarization As Integer 'Polarization to plot(P,S,R)
	TgtType As Integer		'Type of target(ex. Reflectance, Transmittance)
	TgtTol As Single 		'Tolerance
	Out As Integer 			'Output mode(0=Target, 1=Spectra)
	Wmin As Single			'Mimimum wavelength[nm]
	Wmax As Single			'Maximum wavelength[nm]
	Wstep As Single			'Wavelength step[nm]
	nw As Boolean 			'True=newAFalse=append
End Type
Dim TD As TargetData

Dim ColorTable(8) As String	'Color table for plot
Dim ColorAdr As Integer		'Color table address for plot

Dim isFinish As Boolean

Sub Main

	'Initialization fo color table
	InitColorTable

	'Load UDS file & generate CSV file
	If LoadUDS=False Then
		Exit Sub
	End If

	'Set minimum, maximum and step of wavelength to TD structure
	TD.Wmin = Sp.W_begin
	TD.Wmax = Sp.W_end
	TD.Wstep = Sp.W_step

	'Target generation dialogbox
	Do
	If DlgGen = True Then
		Select Case TD.Out
		Case 0 'Target
			GenTgt 'Generate target
			MainActivate
			SendKeys "%a%ow"
			Exit Do
		Case 1 'Spectra
			GenSpec 'Generate "Spectra"
			MainActivate
			If isFinish=False Then
				LineColor ColorTable(ColorAdr)
				ColorAdr = (ColorAdr + 1) Mod 8
				Replot
			ElseIf isFinish=True Then
				Replot
				Exit Do
			End If
			'SendKeys "%a"
		End Select
	Else
		Exit Do
	End If
	Loop
End Sub

'Target generation dialogbox
Function DlgGen As Boolean
	Dim ListArray(2) As String

    isFinish=True
	DlgGen=True

	ListArray(1) = "Reflectance"
	ListArray(2) = "Transmittance"

	Begin Dialog UserDialog 480,224,"Load UDS",.DlgGenFunc ' %GRID:10,7,1,1
		GroupBox 0,0,190,63,"Output to ...",.GroupBox1
		OptionGroup .Group2
			OptionButton 10,21,80,14,"Tar&get",.OptionButton1
			OptionButton 100,21,80,14,"Sp&ectra",.OptionButton2
		CheckBox 10,42,60,14,"Ne&w",.New_CB
		Text 10,77,110,14,"&Type",.Text3,1
		DropListBox 130,70,150,56,ListArray(),.Type_DLB
		Text 290,77,80,14,"To&lerance",.Text9,1
		TextBox 380,70,90,21,.TgtTol_TB
		Text 130,98,60,14,"&Min",.Text1,2
		TextBox 130,112,60,21,.Wmin_TB
		Text 210,98,60,14,"Ma&x",.Text5,2
		TextBox 210,112,60,21,.Wmax_TB
		Text 290,98,60,14,"S&tep",.Text6,2
		TextBox 290,112,60,21,.Wstep_TB
		Text 20,161,100,14,"&AOI[deg]",.Text2,1
		TextBox 130,154,90,21,.AOI_TB
		GroupBox 310,140,160,42,"Polarization",.Pol_GB
		OptionGroup .Group1
			OptionButton 320,161,40,14,"&P",.P_OB
			OptionButton 370,161,40,14,"&S",.S_OB
			OptionButton 420,161,40,14,"&R",.R_OB
		CancelButton 280,196,90,21,.Cancel_PB
		Text 10,119,110,14,"Wavelength[nm]",.Text4,1
		OKButton 380,196,90,21,.OK_PB
		PushButton 30,196,230,21,"Pl&ot Spectra ( Color )",.Plot_PB
	End Dialog
	Dim dlg As UserDialog
	If Dialog(dlg) = 0 Then 'Click cancel button
		DlgGen = False
		Exit Function
	End If
Quit:
    Exit Function
End Function

Function DlgGenFunc%(DlgItem$, Action%, SuppValue%)
	Dim ss As String

    Select Case Action%
    Case 1 ' Dialog box initialization
		'Load of initialization file
		ReadInit
    Case 2 ' Value changing or button pressed
    	If DlgItem$ = "OK_PB" Then
			'Error check
    		If DlgGenErr=False Then
        	    DlgGenFunc% = True 'do not exit the dialog
				Exit Function
    		End If
			'Output to initialization file
    		WriteInit
            DlgGenFunc% = False 'exit the dialog
            isFinish=True
        ElseIf DlgItem$ = "Plot_PB" Then
        	If 	DlgValue("Group2")=0 Then
            	DlgGenFunc% = True 'do not exit the dialog
        	ElseIf DlgValue("Group2")=1 Then
				'Output to initialization file
    			WriteInit
            	DlgGenFunc% = False 'exit the dialog
            	isFinish=False
            End If
        	'Spectravbg
			'GenSpec 'Spectra̐
			'LineColor ColorTable(ColorAdr)
			'ColorAdr = (ColorAdr + 1) Mod 8
			'Replot
            'DlgGenFunc% = True 'not exit the dialog
        'ElseIf DlgItem$ = "Cancel_PB" Then
			't@C̏o
    	'	WriteInit
         '   DlgGenFunc% = False 'exit the dialog
          '  isFinish=True
        End If
    Case 4 ' Focus changed
        Debug.Print "DlgFocus=""";DlgFocus();""""
    Case 6 ' Function key
        If SuppValue And &H100 Then Debug.Print "Shift-";
        If SuppValue And &H200 Then Debug.Print "Ctrl-";
        If SuppValue And &H400 Then Debug.Print "Alt-";
        Debug.Print "F" & (SuppValue And &HFF)
    End Select
End Function

Function DlgGenErr As Boolean
	DlgGenErr=True
	If DTCheck("AOI_TB")<0 Then
		MsgBox "Input number > 0 to AOI."
		DlgGenErr = False
	End If
End Function

'Generate target
Sub GenTgt
	Dim xd() As Single, yd() As Single
	Dim xd_save() As Single, yd_save() As Single
	Dim StrType As String, StrPol As String
	Dim i As Integer

   	ReDim xd(Sp.num), yd(Sp.num, 2)  ' MUST BE 'SINGLE'

   	xd_save = Spectrum_X
   	yd_save = Spectrum_Y

   	For i = 1 To Sp.num Step 1
   		xd(i) = Sp.W_end - Sp.W_step*(Sp.num - i)
   		yd(i, 1) = Sp.Data(i)
   		yd(i, 2) = Sp.Data(i)
   	Next i
   	Spectrum_X = xd   ' Set X values
   	Spectrum_Y = yd   ' Set Y values

   	DataConvert TD.Wmin, TD.Wmax, TD.Wstep

	'Storage Type to StrType
	Select Case TD.TgtType
	Case 0 'Reflectance
		StrType = "R"
	Case 1 'Transmittance
		StrType = "T"
	End Select
	'Storage Polarization to StrPol
	Select Case TD.Polarization
	Case 0 'P
		StrPol = "P"
	Case 1 'S
		StrPol = "S"
	Case 2 'R
		StrPol = "R"
	End Select

   	SpecTargets(TD.nw, StrType, TD.TgtTol, TD.AOI, StrPol)

   	Spectrum_X=xd_save
	Spectrum_Y=yd_save
End Sub

'Generate Spectra
Sub GenSpec
	Dim xd1() As Single, yd1() As Single
	Dim xd2() As Single, yd2() As Single
	Dim xd_save() As Single, yd_save() As Single
	Dim StrType As String, StrPol As String
	Dim FNo As Integer, i As Integer
	Dim FName As String

   	ReDim xd1(Sp.num), yd1(Sp.num, 2)  ' MUST BE 'SINGLE'

	'Storage original spectrum
   	xd_save = Spectrum_X
   	yd_save = Spectrum_Y

	'Change minimum and maximum wavelength
	Select Case TD.TgtType
	Case 0 'Reflectance
	   	For i=1 To Sp.Num Step 1
			xd1(i) = Sp.W_end - Sp.W_step*(Sp.num - i)
			yd1(i,1) = Sp.Data(i)
			yd1(i,2) = 0
		Next i
	Case 1 'Transmittance
	   	For i=1 To Sp.Num Step 1
			xd1(i) = Sp.W_end - Sp.W_step*(Sp.num - i)
			yd1(i,1) = 0
			yd1(i,2) = Sp.Data(i)
		Next i
	End Select
   	Spectrum_X = xd1
   	Spectrum_Y = yd1
   	DataConvert TD.Wmin, TD.Wmax, TD.Wstep
   	xd2 = Spectrum_X
   	yd2 = Spectrum_Y

	'Reset spectrum
   	Spectrum_X=xd_save
	Spectrum_Y=yd_save

   	'Generate CSV
   	FName= MacroDir$() & FNAME_SPEC
	FNo=FreeFile
	Open FName For Output As FNo
		Write #FNo, Sp.Num
		Write #FNo, Sp.Num
		For i = 1 To UBound(xd2) Step 1
			If yd2(i,1)<0 Then
				yd2(i,1)=0
			ElseIf yd2(i,2)<0 Then
				yd2(i,2)=0
			End If
			Debug.Print CStr(xd2(i)) & " " & CStr(yd2(i,1)) & " "  & CStr(yd2(i,2))
			Write #FNo, xd2(i), yd2(i,1), yd2(i,2)
		Next i
	Close FNo

	'Load CSV as Spectra
	FileOpen FName, 1 'Open as Spectra data file
End Sub

Sub InitColorTable
	ColorAdr = 1

	ColorTable(1) = "K"
	ColorTable(2) = "B"
	ColorTable(3) = "G"
	ColorTable(4) = "C"
	ColorTable(5) = "R"
	ColorTable(6) = "M"
	ColorTable(7) = "Y"
	ColorTable(8) = "W"
End Sub


'Load of initialization file
Sub ReadInit
	Dim FName As String, ss As String
	Dim FNo As Integer, i As Integer

	'Load of initialization file
   	FName= MacroDir$() & FNAME_INI
	FNo = FreeFile
	Open FName For Input As FNo
		Input #FNo, TD.AOI, TD.TgtType, TD.TgtTol, TD.Polarization, TD.Out
	Close FNo

	'Set to dialogbox
	DlgText("AOI_TB", CStr(TD.AOI))
	DlgText("TgtTol_TB", CStr(TD.TgtTol))
	DlgValue("Group1", TD.Polarization)
	DlgValue("Group2", TD.Out)
	DlgText("Wmin_TB", CStr(TD.Wmin))
	DlgText("Wmax_TB", CStr(TD.Wmax))
	DlgText("Wstep_TB", CStr(TD.Wstep))
	Select Case Sp.DataType
	Case 0 'Transmittance
		DlgValue("Type_DLB", 1)
	Case 4 'Reflectance
		DlgValue("Type_DLB", 0)
	Case Else 'Other
		DlgValue("Type_DLB", 0)
	End Select
	DlgValue("New_CB", 1)
End Sub

'Output to initialization file
Sub WriteInit
	Dim FName As String, ss As String
	Dim FNo As Integer, i As Integer

	'Get from dialogbox
	TD.AOI = DTCheck("AOI_TB")
	TD.TgtType = DlgValue("Type_DLB")
	TD.TgtTol = DTCheck("TgtTol_TB")
	TD.Polarization = DlgValue("Group1")
	TD.Out = DlgValue("Group2")
	TD.Wmin = DTCheck("Wmin_TB")
	TD.Wmax = DTCheck("Wmax_TB")
	TD.Wstep = DTCheck("Wstep_TB")
	TD.nw = DlgValue("New_CB")

	If TD.TgtType = 0 Then 'R
		Sp.DataType = 4
	ElseIf TD.TgtType = 1 Then
		Sp.DataType = 0
	End If

	'Output to initialization file
   	FName= MacroDir$() & FNAME_INI
	FNo = FreeFile
   	Open FName For Output As FNo
		Write #FNo, TD.AOI, TD.TgtType, TD.TgtTol, TD.Polarization, TD.Out
	Close FNo
End Sub

Function DTCheck(ss As String ) As Single
	If DlgText(ss)="" Then
		DTCheck=0
	Else
		DTCheck = DlgText(ss)
	End If
End Function

'========== Functions to read UDS file ==========
'Read UDS file and generate CSV file
Function LoadUDS
    Dim i%
    Dim FName As String

    LoadUDS = True

    On Error GoTo Load_Uds_error

   	FName=GetFilePath$(,"UDS",,"Read of UDS file",1)

   	If FName="" Then
		LoadUDS=False
		Exit Function
   	End If

    ReadUDS FName, Sp 'Read UDS file

    If Sp.DataType = 0 Then
    ElseIf Sp.DataType = 4 Then
    Else
        MsgBox "This data is not UDS.", 48, "Data error"
        LoadUDS = False
        Exit Function
    End If

    'LoadUDS = Sp.Num
	Exit Function
Load_Uds_error: 'error
    Reset 'reset file
    MsgBox Error$, 48, " error code " & Str$(Err) 'message
    LoadUDS = False 'Stop
    Exit Function
End Function

'Core function to read UDS
'Comment, Operator, date are not read
Sub ReadUDS(File_Path As String, Sp As SpFileData)
	Dim FN As Integer
    Dim strD As String
    Dim strModel As String
    Dim dD As Double
    Dim i As Long
    Dim FSize As Long
    Dim OFS1 As Long
    Dim OFS2 As Long
    Dim buff() As Double
    Dim test01

    With Sp
    	FN = FreeFile()

       	Open File_Path For Binary Access Read As #FN

       	.File_Path = File_Path
       	strD = "12345678"
       	Get #FN, 1, strD
		If strD = "IIHIITAG" Then ' UDS File
	    	FSize = LOF(FN)
        	ReDim buff(FSize / 8) 'Tentative data size
        	'&H08..&H0B ?

        	'&H0C..&H0F  OFFSET 1
        	Get #FN, &HD, OFS1

        	'If mStrRead(FN, .Title) < 0 Then GoTo ReadError
        	'Comment
        	'If mStrRead(FN, .Comment) < 0 Then GoTo ReadError
        	'Operatpor
        	'If mStrRead(FN, strD) < 0 Then GoTo ReadError
        	'Date
        	'If mStrRead(FN, strD) < 0 Then GoTo ReadError

        	'Model
        	'If mStrRead(FN, strModel) < 0 Then GoTo ReadError
			Do
		   		mStrRead(FN, strModel)
	           	If strModel =     "85455249484814096149170140245140245147120140118" Then 'U-4100
					Exit Do
				ElseIf strModel = "85455248484814096149170140245140245147120140118" Then 'U-4000
					Exit Do
        	   	End If
        	Loop

           	'ROM ver.
           	If mStrRead(FN, strD) < 0 Then GoTo ReadError
			'Sirial No.
           	If strModel =     "85455249484814096149170140245140245147120140118" Then 	'U-4100
				If mStrRead(FN, strD) < 0 Then GoTo ReadError
           	ElseIf strModel = "85455248484814096149170140245140245147120140118" Then 	'U-4000
            	If mStrRead(FN, strD) < 0 Then GoTo ReadError
           	End If

            '?  2.0 or 4.0
            Get #FN, , dD
            'Voltage
            If mStrRead(FN, strD) < 0 Then GoTo ReadError
			'strD = ""
            '?  200.0 or 500.0
            Get #FN, , dD
            Debug.Print "200/500=" & dD
            '? 0.0  ( 0 (8 byte) )
            Get #FN, , dD
            '? 0.0  ( 0 (8 byte) )
            Get #FN, , dD
            'Base line(ex:User 1=2131761872223249)
            If mStrRead(FN, strD) < 0 Then GoTo ReadError
            'Wavelength to change illuminant
            Get #FN, , dD
            'Wavelength step
            Get #FN, , dD
            Sp.W_step = dD
            'Start wavelengthiMaximum wavelengthj
            Get #FN, , dD
            Sp.W_end = dD
            'Spectral data
            .Num = 0
            Do
                If EOF(FN) Then
                	Debug.Print ".Num=" & .Num
					GoTo ReadError
				End If
                Get #FN, , dD
                Debug.Print ".Num(" & .Num & "):" & dD
                If .Num= 599 Then
	                Debug.Print ".Num(" & .Num & "):" & dD
                End If
                If dD = 60# Or dD=3600# Then
	                Debug.Print dD
                	Debug.Print ".Num=" & .Num
                	Exit Do
                End If
                .Num = .Num + 1
                buff(.Num) = dD
            Loop
            Debug.Print ".Num=" & .Num

            'Start wavelengthiMaximum wavelengthjagain ?
            Get #FN, , dD
            'Scan speed
            Get #FN, , dD
            'Start wavelengthiMaximum wavelengthjagain ?
            Get #FN, , dD
            'Minimum wavelength
            Get #FN, , dD
            .W_begin = dD

            If OFS1 + 59 > FSize - 3 Then
			    Debug.Print ".Num=" & .Num
            	GoTo ReadError
            End If
            Get #FN, OFS1 + 59, OFS2
            If OFS2 + 203 > FSize - 1 Then GoTo ReadError
            'Data mode
            strD = "  "
            Get #FN, OFS2 + 203, strD

            Select Case strD
            Case "%R"
                Sp.DataType = 4
            Case "%T"
                Sp.DataType = 0
            Case Else
                Sp.DataType = -1
            End Select
        Else  ' Unknown
            GoTo ReadError
        End If

        ReDim Sp.Data(Sp.Num)
        For i = 1 To Sp.Num
            Sp.Data(i) = buff(.Num - i + 1)
        Next i
    End With
    Close #FN
    Exit Sub
ReadError:
     Close #FN
     MsgBox ".UDS file read error !!"
     End
End Sub

'Read until Hexadecimal "00"
Function mStrRead(FN As Integer, strBuff As String) As Long
	'Read until Hexadecimal "00"
	'Return the number of read byte
    Dim c As Byte
	Dim count As Integer
	count = 0

    strBuff = ""
    mStrRead = 0
    Do While Not EOF(FN)
        Get #FN, , c
        'Debug.Print Hex(c)
        If c = 0 Then
        'If c = Hex(0) Then
        '	Debug.Print "str=" & strBuff
        	Exit Function
        End If
        strBuff = strBuff & CStr(c)
        mStrRead = mStrRead + 1
    Loop
    mStrRead = -1 ' Unexpected end of file
End Function
